import { Signal } from 'solid-js';
import PouchDB from 'pouchdb';
import indexeddb from 'pouchdb-adapter-indexeddb';

import { getSettings } from '../db/settings';
import { delay } from '../lib/delay';

PouchDB.plugin(indexeddb);

export const compactDb = async (p: { signal: Signal<any>; params: any }) => {
  const { signal, params } = p;
  const [, setStatus] = signal;
  console.log({ caller: 'compactDb', p });

  const db = new PouchDB('_dyomedea_', {
    auto_compaction: true,
    adapter: 'indexeddb',
  });
  const dbinfo = await db.info();
  const tasks = PouchDB.activeTasks.list();

  setStatus({ status: 'starting', db: '_dyomedea_', dbinfo, tasks });

  const timerId = setInterval(async () => {
    const dbinfo = await db.info();
    const tasks = PouchDB.activeTasks.list();
    if (tasks.length > 0) {
      setStatus({ status: 'ongoing', db: 'dyomedea', dbinfo, tasks });
    } else {
      setStatus({ status: 'terminated', db: 'dyomedea', dbinfo, tasks });
      clearInterval(timerId);
    }
  }, 1000);

  // db.compact();

  const openIDb = (name: string) =>
    new Promise((resolve, reject) => {
      const iDb = indexedDB.open(name);
      iDb.onerror = (event: any) => {
        console.error({
          caller: 'compactDb',
          message: 'open db error',
          target: event.target,
        });
        reject(event.target.errorCode);
      };

      iDb.onsuccess = (event: any) => {
        console.log({
          caller: 'compactDb',
          message: 'open db',
          target: event.target,
        });
        resolve(event.target.result);
      };
    });

  const getAll = (store: any) =>
    new Promise((resolve, reject) => {
      const request = store.getAll();
      request.onerror = (event: any) => {
        console.error({
          caller: 'compactDb',
          message: 'getAll error',
          target: event.target,
        });
        reject(event.target.errorCode);
      };

      request.onsuccess = (event: any) => {
        console.log({
          caller: 'compactDb',
          message: 'getAll',
          target: event.target,
        });
        resolve(event.target.result as any[]);
      };
    });

  const iDb = (await openIDb('_pouch__dyomedea_')) as IDBDatabase;

  const bySequence = iDb.transaction('docs', 'readonly');
  const store = bySequence.objectStore('docs');
  console.log({
    caller: 'compactDb',
    message: 'transaction opened',
    bySequence,
    store,
  });

  const docs = (await getAll(store)) as any[];
  //   setStatus({
  //     status: 'docs',
  //     db: 'dyomedea',
  //     docs,
  //   });

  docs.forEach(async (doc) => {
    const { id, rev, deleted } = doc;
    if (deleted !== 0) {
      //     const purge = await db.purge(id, rev);
      console.log({
        caller: 'compactDb',
        message: 'purging',
        id,
        rev,
        doc,
        //       purge,
      });
      await delay(1000);
    }
  });
};
